//
// hepml.hpp
// HepML API public include header
// Version 0.2
// part of LCG MCDB project, http://mcdb.cern.ch
// Monte Carlo Data Base
// Sergey Belov <Sergey.Belov@cern.ch>, 2006-2007
//

#ifndef __HEPML_HPP__
#define __HEPML_HPP__ 1


#include <string>
#include <vector>

static const char* HEPML_API_VERSION = "0.2";

namespace mcdb
{

using std::string;
using std::vector;


class Article;
class File;
class Author;
class Cut;
class CutSet;
class Generator;
class Model;
class Process;
class Subprocess;
struct Beam;
struct CrossSection;
struct Energy;
struct QcdCoupling;
struct MathNotation;


// MC generator information
class Generator {
 public:
    Generator();
    virtual ~Generator();
    string& name();
    string& name(const string&);
    string& version();
    string& version(const string&);
    string& homepage();
    string& homepage(const string&);
    string& description();
    string& description(const string&);
 private:
    string name_;
    string version_;
    string homepage_;
    string description_;
}; // class Generator


// Mathematical notation structure
struct MathNotation {
    string plain;
    string html;
    string latex;
    string mathml;
};

// Physical model
class Model {
 public:
    Model();
    virtual ~Model();
    class Parameter;
    typedef vector<Parameter> ParametersVector;

    string& name();
    string& name(const string&);
    string& description();
    string& description(const string&);
    ParametersVector& parameters();
    ParametersVector& parameters(const ParametersVector&);

    class Parameter {
     public:
        Parameter();
	Parameter(const string& name, const string& value);
        virtual ~Parameter();
        string& name();
        string& name(const string&);
        string& value();
        string& value(const string&);
	string& description();
	string& description(const string&);
        MathNotation& mathNotation();
        MathNotation& mathNotation(const MathNotation&);
     private:
        string name_;
        string value_;
        string description_;
        MathNotation mathNotation_;
    };
 private:
    string name_;
    string description_;
    ParametersVector parameters_;
}; // class Model


struct Pdf {
    string name;
    string version;
    int pdfLibSet;
    int pdfLibGroup;
    int lhaPdfSet;
    int lhaPdfMember;
    string lhaPdfFile;
    
    Pdf() :
            pdfLibSet(0), pdfLibGroup(0),
                      lhaPdfSet(0), lhaPdfMember(0) {}
};


struct QcdCoupling {
    double lambda;
    int   nFlavours;
    int   nLoopsAlpha;
    
    QcdCoupling() :
            lambda(0), nFlavours(0), nLoopsAlpha(0) {}
};


struct CrossSection  {
    enum Unit { b=0, mb=1, mub=2, nb=3, pb=4, fb=5, ab=6 };

    CrossSection() :
            value(0), unit(pb), errorPlus(0), errorMinus(0)  {}
    CrossSection(double val, Unit u) :
            value(val), unit(u), errorPlus(0), errorMinus(0) {}
    // give corresponding string value for measurement unit
    string unitStr();
    static string unitStr(Unit u);

    double value;
    Unit   unit;
    double errorPlus;
    double errorMinus;
    
};


struct Beam {
    struct Particle {
        string name;
        int    kfCode;
        
        Particle() :
                name(""), kfCode(0) {}
        Particle(string n, int code=0) :
                name(n), kfCode(code) {}
    };
    
    struct Energy {
        enum Unit { eV=0, keV=1, MeV=2, GeV=3, TeV=4 };
        double value;
        Unit   unit;

        Energy() :
                value(0), unit(GeV) {}
        Energy(double val, Unit u) :
                value(val), unit(u) {}
        // give corresponding string value for measurement unit
	string unitStr();
        static string unitStr(Unit);
    };

    Particle     particle;
    Energy       energy;
    Pdf          pdf;
    QcdCoupling  qcdCoupling;
    
    Beam() {}
};


typedef vector<Subprocess> SubprocessVector;

// Physical process description
class Process {
 public:
    Process();
    virtual ~Process();
    
    Beam&   beam1();
    Beam&   beam1(const Beam&);
    Beam&   beam2();
    Beam&   beam2(const Beam&);
    string& finalState();
    string& finalState(const string&);
    MathNotation& finalStateNotation();
    MathNotation& finalStateNotation(const MathNotation&);
    QcdCoupling& qcdCoupling();
    QcdCoupling& qcdCoupling(const QcdCoupling&);
    CrossSection& crossSection();
    CrossSection& crossSection(const CrossSection&);
    SubprocessVector& subprocesses();
    SubprocessVector& subprocesses(const SubprocessVector&);
 private:
     Beam   beam1_;
     Beam   beam2_;
     string finalState_;
     MathNotation finalStateNotation_;
     QcdCoupling qcdCoupling_;
     CrossSection crossSection_;
     SubprocessVector subprocesses_;
}; // class Process


class Subprocess {
 public:
     Subprocess();
     virtual ~Subprocess();
     string& notation();
     string& notation(const string&);
     CrossSection& crossSection();
     CrossSection& crossSection(const CrossSection&);
     MathNotation& factorisationScale();
     MathNotation& factorisationScale(const MathNotation&);
     MathNotation& renormalisationScale();
     MathNotation& renormalisationScale(const MathNotation&);
     int& cutsetId();
     int& cutsetId(int);
 private:
     string notation_;
     CrossSection crossSection_;
     MathNotation factorisationScale_;
     MathNotation renormalisationScale_;
     int cutsetId_;
};

// Information of samples' author
class Author {
 public:
    Author();
    ~Author();
    string& firstName();
    string& firstName(const string&);
    string& lastName();
    string& lastName(const string&);
    string& email();
    string& email(const string&);
    string& experiment();
    string& experiment(const string&);
    string& experimentGroup();
    string& experimentGroup(const string&);
    string& organization();
    string& organization(const string&);
 private:
    string firstName_;
    string lastName_;
    string email_;
    string experiment_;
    string experimentGroup_;
    string organization_;
}; // class Author


// Applied cut
class Cut {
 public:
    // Logic of applied cut restriction
    enum Logic { include_region=0, exclude_region=1 };

    Cut();
    virtual ~Cut();
    string& object();
    string& object(const string&);
    MathNotation& objectNotation();
    MathNotation& objectNotation(const MathNotation&);
    string& minValue();
    string& minValue(const string&);
    string& maxValue();
    string& maxValue(const string&);
    Logic& logic();
    Logic& logic(const Logic&);
    int& cutsetId();
    int& cutsetId(int id);
 private:
    string object_;
    MathNotation objectNotation_;
    string minValue_;
    string maxValue_;
    Logic logic_;
    int cutsetId_;
}; // class Cut

typedef vector<Cut> CutsVector;


// Event sample/metadata file description
class File {
 public:
    // Possible file types in MCDB:
    // 'data' - sample file, 'meta' - file containing some metadata,
    // 'other' - something else
    enum Type { other=0, data=1, metadata=2, generator_config=3, generator_binary=4 };
    
    
    struct Checksum {
        Checksum() : type(unknown) {}

        enum Type { unknown=0, mda5=1, sha1=2, crc32=3 };
        string typeStr();
	static string typeStr(Type);

        Type   type;
        string value;
    }; // struct File::Checksum
    
    
    File();
    virtual ~File();

    File& operator=(const File&);
    
    Type& type();
    Type& type(const Type& type);
    string typeStr();
    static string typeStr(Type);
    int& id();
    int& id(int id);
    unsigned long& eventsNumber();
    unsigned long& eventsNumber(unsigned long number);
    unsigned long& size();
    unsigned long& size(unsigned long size);
    Checksum& checksum();
    Checksum& checksum(const Checksum&);
    CrossSection& crossSection();
    CrossSection& crossSection(const CrossSection&);
    string& comments();
    string& comments(const string&);
    vector<string>& paths();
    vector<string>& paths(const vector<string>&);
    vector<string> findPaths(const string& substr);
    bool findSubstrInName(const string& substr);
    bool findSubstrInPath(const string& substr);
    bool hasName(const string& nameToFind);
    string getCastorPath();
  private:
    Type type_;
    int id_;
    unsigned long eventsNumber_;
    unsigned long size_;
    Checksum checksum_;
    CrossSection crossSection_;
    string comments_;
    vector<string> paths_;
 public:
     
    static string getNameFromPath(const string &path);
    static bool isPlainName(const string& url);
}; // class File

typedef vector<File> FilesVector;

struct ExperimentGroup {
	string experiment;
	string group;
        string responsiblePerson;
	string description;
        
        ExperimentGroup() {}
        ExperimentGroup(const string& exp, 
                        const string& grp,
                        const string& descr);
};

// Description for a set of event samples 
class Article {
 public:
    Article();
    virtual ~Article();
    int& id();
    int& id(int id);
    string& title();
    string& title(const string&);
    string& abstract();
    string& abstract(const string&);
    string& comments();
    string& comments(const string&);
    ExperimentGroup& experimentGroup();
    ExperimentGroup& experimentGroup(const ExperimentGroup&);
    vector<Author>& authors();
    vector<Author>& authors(const vector<Author>&);
    const string postDate();
    Process& process();
    Process& process(const Process&);
    Generator& generator();
    Generator& generator(const Generator&); 
    Model& model();
    Model& model(const Model&);
    CutsVector& cuts();
    CutsVector& cuts(const CutsVector&);
    vector<File>& files();
    vector<File>& files(const vector<File>&);
    vector<string>& relatedPapers();
    vector<string>& relatedPapers(const vector<string>&);
 private:
    string title_;
    string abstract_;
    string comments_;
    ExperimentGroup experimentGroup_;
    string documentId_;
    string databaseId_;
    string postDate_;
    int id_;
    vector<File> files_;
    vector<string> relatedPapers_;
    Generator generator_; 
    Model model_;
    Process process_;
    CutsVector cuts_;
    vector<Author> authors_;
}; // Class Article


} // namespace mcdb

#endif /* __HEPML_HPP__ */

