//
// MCDB API: HepmlWriter.cpp
// LCG MCDB project, Monte Carlo Data Base
// http://mcdb.cern.ch
// 
// Sergey Belov <Sergey.Belov@cern.ch>, 2007
//


#include <map>
#include <sstream>
#include "hepml.hpp"
#include "XmlWriter.hpp"

/*
* Declaration part
*/

namespace hepml {
    std::string articleToXml(mcdb::Article&, int indentLevel=0, bool addXmlHeader=false);
}

namespace hepml {
    static const char* XMLNS               = "http://mcdb.cern.ch/hepml/0.2/";
    static const char* XMLNS_XSI           = "http://www.w3.org/2001/XMLSchema-instance";
    static const char* SCHEMA_LOCATION_XSI = "http://mcdb.cern.ch/hepml/0.2/ http://mcdb.cern.ch/hepml/0.2/hepml.xsd";
}


namespace hepml {

using namespace hepml::xml;
    
class ArticleXml {
    public:
        explicit ArticleXml(const mcdb::Article& a) : a_(a) {}
        ~ArticleXml() {}
        XmlElement toXml();
        XmlElement filesToXml();
    private:
        XmlElement descriptionToXml();
        XmlElement cutsToXml();
        XmlElement authorsToXml();
        
        mcdb::Article a_;
        
};


class FileXml {
    public:
        explicit FileXml(const mcdb::File& f) : f_(f) {}
        XmlElement toXml();
    private:
        mcdb::File f_;
};

class ExperimentGroupXml {
    public:
        explicit ExperimentGroupXml(const mcdb::ExperimentGroup& eg)
            : eg_(eg) {}
        XmlElement toXml();
    private:
        mcdb::ExperimentGroup eg_;
};

class GeneratorXml {
    public:
        explicit GeneratorXml(const mcdb::Generator& g) : g_(g) {}
        XmlElement toXml();
    private:
        mcdb::Generator g_;
};


class ModelXml {
    public:
        explicit ModelXml(const mcdb::Model& m) : m_(m) {}
        XmlElement toXml();
    private:
        class ModelParameterXml {
            public:
                explicit ModelParameterXml(const mcdb::Model::Parameter p) : p_(p) {}
                XmlElement toXml();
            private:
                mcdb::Model::Parameter p_;
        };

        XmlElement parametersToXml();
        mcdb::Model m_;
};

class ProcessXml {
    public:
        explicit ProcessXml(const mcdb::Process& p) : p_(p) {}
        XmlElement toXml();
    private:
        XmlElement subprocessesToXml();
        mcdb::Process p_;
};

class SubprocessXml {
    public:
        explicit SubprocessXml(const mcdb::Subprocess& s) : s_(s) {}
        XmlElement toXml();
    private:
        mcdb::Subprocess s_;
};

class BeamXml {
    public:
        explicit BeamXml(const mcdb::Beam& b) : b_(b) {}
        XmlElement toXml();
    private:
        mcdb::Beam b_;
};

class QcdCouplingXml {
    public:
        explicit QcdCouplingXml(const mcdb::QcdCoupling& c) : c_(c) {}
        XmlElement toXml();
    private:
        mcdb::QcdCoupling c_;
};

class PdfXml {
    public:
        explicit PdfXml(const mcdb::Pdf& p) : p_(p) {}
        XmlElement toXml();
    private:
        mcdb::Pdf p_;
};

class MathNotationXml {
    public:
        explicit MathNotationXml(const mcdb::MathNotation& n) : n_(n) {}
        XmlElement toXml();
    private:
        mcdb::MathNotation n_;
};

class CrossSectionXml {
    public:
        explicit CrossSectionXml(const mcdb::CrossSection& cs) : cs_(cs) {}
        XmlElement toXml();
    private:
        mcdb::CrossSection cs_;
};

class CutXml {
    public:
        explicit CutXml(const mcdb::Cut& c) : c_(c) {}
        XmlElement toXml();
    private:
        mcdb::Cut c_;
};

class AuthorXml {
    public:
        explicit AuthorXml(const mcdb::Author& a) : a_(a) {}
        XmlElement toXml();
    private:
        mcdb::Author a_;
};

} //namespace hepml




/*
* Implementation part
*/


namespace hepml {
    

/** Convert value of any type to string
 * @param t value of template type T
 * @return std::string contains transformed value
 */
template <class T>
inline std::string toString(const T& t)
{
    std::stringstream ss;
    ss << t;
    return ss.str();
}

XmlElement ArticleXml::toXml()
{
    XmlElement e ( "samples" );
    e.addAttribute( "xmlns", XMLNS );
    e.addAttribute( "xmlns:xsi", XMLNS_XSI );
    e.addAttribute( "xsi:schemaLocation", SCHEMA_LOCATION_XSI );
    
    XmlSequence s;
    
    s.addElement( filesToXml() );
    s.addElement( descriptionToXml() );
    
    e.setContent( s );
    return e;
}

XmlElement ArticleXml::filesToXml()
{
    XmlElement e ( "files" );
    XmlSequence s;
    
    mcdb::FilesVector &vect = a_.files();
    mcdb::FilesVector::iterator i = vect.begin(),
                                end = vect.end();
    for ( ; i != end; ++i ){
        s.addElement( FileXml(*i).toXml() );
    }
        
    
    e.setContent( s );
    return e;
}

XmlElement ArticleXml::descriptionToXml()
{
    XmlElement e ( "description" );
    XmlSequence s;
    
    s.addElement( XmlElement( "title", a_.title() ) );
    s.addElement( XmlElement( "abstract", a_.abstract() ) );
    s.addElement( XmlElement( "authorComments", a_.comments() ) );
    
    s.addElement( ExperimentGroupXml(a_.experimentGroup()).toXml() );
    
    s.addElement( GeneratorXml(a_.generator()).toXml() );
    s.addElement( ModelXml(a_.model()).toXml() );
    s.addElement( ProcessXml(a_.process()).toXml() );
    
    s.addElement( cutsToXml() );
    s.addElement( authorsToXml() );
    
    
    if ( a_.relatedPapers().size() ) {
        string relatedPapers;
        std::vector<string>::iterator i = a_.relatedPapers().begin(),
                                      end = a_.relatedPapers().end();
        for ( ; i != end; ++i) {
            if (! relatedPapers.empty() ) {
                relatedPapers += "";
            }
            relatedPapers += *i;
        }
        s.addElement( XmlElement( "relatedPapers", relatedPapers ) );
    }
    
    
    e.setContent( s ); 
    
    return e;
}

// Forms list of cutsToXml.
// Cut sets in output are in order of appearance
// in cuts() array in mcdb::Article
XmlElement ArticleXml::cutsToXml()
{
    using mcdb::CutsVector;
    
    XmlElement e ( "cuts" );
    XmlSequence s;
    
                
    typedef std::map<int, XmlSequence> CutsetMap;
    CutsetMap cutSets;
    {
        CutsVector& cuts = a_.cuts();
        CutsVector::iterator i = cuts.begin(), end = cuts.end();
        for ( ; i != end; ++i) {
            int id = i->cutsetId();
            cutSets[id].addElement( CutXml(*i).toXml() );
        }
    }
    
    
    {
        CutsetMap::iterator i = cutSets.begin(), end = cutSets.end();
        for ( ; i != end; ++i) {
            int id = i->first;
            XmlElement cutset ( "cutSet", i->second );
            cutset.addAttribute( "cutset_id", toString(id) );
            
            s.addElement ( cutset );
        }
    }
    
    
    e.setContent( s );
    return e;
}


XmlElement ArticleXml::authorsToXml()
{
    XmlElement e ( "authors" );
    XmlSequence s;
    std::vector<mcdb::Author>::iterator i = a_.authors().begin(),
                                        end = a_.authors().end();
    for ( ; i != end; ++i ) {
        s.addElement( AuthorXml(*i).toXml() );
    }
    
    e.setContent( s );
    return e;
}


XmlElement FileXml::toXml()
{
    XmlElement e ( "file" );
    XmlSequence s;
    
    e.addAttribute( "type", f_.typeStr() );
    s.addElement( XmlElement( "eventsNumber", toString(f_.eventsNumber()) ) );
    s.addElement( CrossSectionXml(f_.crossSection()).toXml() );
    if ( f_.size() != 0 ) {
        s.addElement( XmlElement( "fileSize", toString(f_.size()) ) );
    }
    if ( ! f_.checksum().value.empty() ) {
        XmlElement csum ( "checksum", f_.checksum().value );
        csum.addAttribute( "type", f_.checksum().typeStr() );
        s.addElement( csum );
    }
    if ( ! f_.comments().empty() ) {
        s.addElement( XmlElement( "comments", f_.comments() ) );
    }
    
    XmlElement location ( "location" );
    {
        XmlSequence paths;
        std::vector<string>::iterator i = f_.paths().begin(),
                                      end = f_.paths().end();
        for ( ; i != end; ++i ) {
            paths.addElement( XmlElement( "path", *i )  );
        }
        location.setContent( paths );
    }
    s.addElement( location );

    e.setContent( s );
    return e;
}


XmlElement ExperimentGroupXml::toXml()
{
    XmlElement e( "experimentGroup" );
    XmlSequence s;
    s.addElement( XmlElement( "experiment", eg_.experiment ) );
    s.addElement( XmlElement( "group", eg_.group ) );
    if ( ! eg_.responsiblePerson.empty() ) {
        s.addElement( XmlElement( "responsiblePerson", eg_.responsiblePerson ) );
    }
    s.addElement( XmlElement( "description", eg_.description ) );
    
    e.setContent( s );
    return e;
}



XmlElement GeneratorXml::toXml()
{
    XmlElement e ( "generator" );
    XmlSequence s;
    
    s.addElement( XmlElement( "name", g_.name() ) );
    s.addElement( XmlElement( "version", g_.version() ) );
    s.addElement( XmlElement( "homepage", g_.homepage() ) );
    s.addElement( XmlElement( "description", g_.description() ) );
    
    e.setContent( s );
    return e;
}


XmlElement ModelXml::toXml()
{
    XmlElement e ( "model" );
    XmlSequence s;
    
    s.addElement( XmlElement( "name", m_.name() ) );
    s.addElement( XmlElement( "description", m_.description() ) );
    s.addElement( parametersToXml() );
    
    
    e.setContent( s );
    return e;
}


XmlElement ModelXml::parametersToXml()
{
    XmlElement e ( "parameters" );
    XmlSequence s;
    
    mcdb::Model::ParametersVector::iterator i = m_.parameters().begin(),
                                            end = m_.parameters().end();
    for ( ; i != end; ++i ) {
        s.addElement( ModelXml::ModelParameterXml( *i ).toXml() );
    }
    
    e.setContent( s );
    return e;
}


XmlElement ModelXml::ModelParameterXml::toXml()
{
    XmlElement e ( "parameter" );
    XmlSequence s;
    
    s.addElement( XmlElement( "name", p_.name() ) );
    s.addElement( XmlElement( "value", p_.value() ) );
    s.addElement( MathNotationXml(p_.mathNotation()).toXml() );
    s.addElement( XmlElement( "description", p_.description() ) );

    e.setContent( s );
    return e;
}


XmlElement ProcessXml::toXml()
{
    XmlElement e ( "process" );
    XmlSequence s;
    
    XmlElement beam1 ( BeamXml(p_.beam1()).toXml() );
    XmlElement beam2 ( BeamXml(p_.beam2()).toXml() );
    beam1.name( "beam1" );
    beam2.name( "beam2" );
    
    s.addElement( beam1 );
    s.addElement( beam2 );
    
    {
        XmlElement finalState ( "finalState" );
        XmlSequence sf;
        sf.addElement( XmlElement( "state", p_.finalState() ) );
        sf.addElement( MathNotationXml(p_.finalStateNotation()).toXml() );
        finalState.setContent( sf );
        s.addElement( finalState );
    }
    
    s.addElement( QcdCouplingXml(p_.qcdCoupling()).toXml() );
    s.addElement( CrossSectionXml(p_.crossSection()).toXml() );
    
    s.addElement( subprocessesToXml() );
    
    e.setContent( s );
    return e;
}


XmlElement ProcessXml::subprocessesToXml()
{
    XmlElement e ( "subprocesses" );
    XmlSequence s;
    
    mcdb::SubprocessVector::iterator i = p_.subprocesses().begin(),
                                     end = p_.subprocesses().end();
    for ( ; i != end; ++i) {
        s.addElement( SubprocessXml(*i).toXml() );
    }

    e.setContent( s );
    return e;
}

XmlElement SubprocessXml::toXml()
{
    XmlElement e ( "subprocess" );
    XmlSequence s;
    
    s.addElement( XmlElement("notation", s_.notation()) );
    s.addElement( CrossSectionXml(s_.crossSection()).toXml() );
    
    XmlElement factScale ( MathNotationXml( s_.factorisationScale() ).toXml() );
    XmlElement renormScale ( MathNotationXml( s_.renormalisationScale() ).toXml() );
    factScale.name( "FactorisationScale" );
    renormScale.name( "RenormalisationScale" );
    
    s.addElement( factScale );
    s.addElement( renormScale );
    
    if ( s_.cutsetId() ) {
        e.addAttribute( "cutset_id", toString(s_.cutsetId()) );
    }
    
    e.setContent( s );
    return e;
}

XmlElement BeamXml::toXml()
{
    XmlElement e;
    XmlSequence s;
    
    XmlElement particle ( "particle", b_.particle.name );
    particle.addAttribute( "KFcode", toString(b_.particle.kfCode) );
    s.addElement( particle );
    
    XmlElement energy ( "energy", toString(b_.energy.value) );
    energy.addAttribute( "unit", b_.energy.unitStr() );
    
    s.addElement( energy );
    s.addElement( QcdCouplingXml(b_.qcdCoupling).toXml() );
    s.addElement( PdfXml(b_.pdf).toXml() );
    
    e.setContent( s );
    return e;
}

XmlElement QcdCouplingXml::toXml()
{
    XmlElement e ( "QCDCoupling" );
    XmlSequence s;
    s.addElement( XmlElement( "Lambda", toString(c_.lambda) ) );
    s.addElement( XmlElement( "NFlavours", toString(c_.nFlavours) ) );
    s.addElement( XmlElement( "NLoopsAlphaS", toString(c_.nLoopsAlpha) ) );

    e.setContent( s );
    return e;
}



XmlElement PdfXml::toXml()
{
    XmlElement e ( "pdf" );
    
    if ( ! p_.name.empty() ) {
        e.addAttribute( "name", p_.name );
    }
    if ( ! p_.version.empty() ) {
        e.addAttribute( "version", p_.version );
    }
    if ( p_.pdfLibSet != 0 ) {
        e.addAttribute( "PDFLIBset", toString(p_.pdfLibSet) );
    }
    if ( p_.pdfLibGroup != 0 ) {
        e.addAttribute( "PDFLIBgroup", toString(p_.pdfLibGroup) );
    }
    if ( p_.lhaPdfSet != 0 ) {
        e.addAttribute( "LHAPDFset", toString(p_.lhaPdfSet) );
    }
    if ( p_.lhaPdfMember != 0 ) {
        e.addAttribute( "LHAPDFmember", toString(p_.lhaPdfMember) );
    }
    if ( ! p_.lhaPdfFile.empty() ) {
        e.addAttribute( "LHAPDFfile", p_.lhaPdfFile );
    }
    
    return e;
}


XmlElement MathNotationXml::toXml()
{
    XmlElement e ( "notation" );
    XmlSequence s;
    
    s.addElement( XmlElement( "plain", n_.plain ) );
    if ( ! n_.html.empty() ) {
        s.addElement( XmlElement( "HTML", n_.html ) );
    }
    if ( ! n_.latex.empty() ) {
        s.addElement( XmlElement( "Latex", n_.latex ) );
    }
    if ( ! n_.mathml.empty() ) {
        s.addElement( XmlElement( "MathML", n_.mathml ) );
    }
    
    e.setContent( s );
    return e;
}


XmlElement CrossSectionXml::toXml()
{
    XmlElement e ( "crossSection", toString( cs_.value ) );
    
    e.addAttribute( "unit", cs_.unitStr() );
    
    if ( cs_.errorPlus != 0 ) {
        e.addAttribute( "errorPlus", toString(cs_.errorPlus) );
    }
    if ( cs_.errorMinus != 0 ) {
        e.addAttribute( "errorMinus", toString(cs_.errorMinus) );
    }
    
    return e;
}


namespace {
    
    string cutLogicToStr(mcdb::Cut::Logic l)
    {
        if ( l == mcdb::Cut::exclude_region ) {
            return "exclude_region";
        } else {
                return "include_region";
        }
    }
    
}

XmlElement CutXml::toXml()
{
    XmlElement e ( "cut" );
    XmlSequence s;
    
    XmlElement cutObject ( "object" );
    {
        XmlSequence objContent;
        objContent.addElement( XmlElement( "name", c_.object() ) );
        objContent.addElement( MathNotationXml(c_.objectNotation()).toXml() );
        
        cutObject.setContent( objContent );
    }
    
    s.addElement( cutObject );
    s.addElement( XmlElement( "minValue", c_.minValue() ) );
    s.addElement( XmlElement( "maxValue", c_.maxValue() ) );
    s.addElement( XmlElement( "logic",  cutLogicToStr(c_.logic()) ) );
    
    e.setContent( s );
    return e;
}


XmlElement AuthorXml::toXml()
{
    XmlElement e ("author");
    XmlSequence s;
    
    s.addElement( XmlElement( "firstName", a_.firstName() ) );
    s.addElement( XmlElement( "lastName", a_.lastName() ) );
    s.addElement( XmlElement( "email", a_.email() ) );
    if ( ! a_.experiment().empty() ) {
        s.addElement( XmlElement( "experiment", a_.experiment() ) );
    }
    if ( ! a_.experimentGroup().empty() ) {
        s.addElement( XmlElement( "group", a_.experimentGroup() ) );
    }
    s.addElement( XmlElement( "organization", a_.organization() ) );
    
    e.setContent( s );
    return e;
}



std::string articleToXml(mcdb::Article& a, int indentLevel, bool addXmlHeader)
{
    std::string result;
    
    if ( addXmlHeader ) {
        result += XML_HEADER + string(LINE_FEED_CHAR);
    }
    
    result += ArticleXml(a).toXml().toXmlString(indentLevel);
    
    return result;
}


} // namespace hepml

