//
// MCDB API: lhaef.hpp
// LCG MCDB project, Monte Carlo Data Base
// http://mcdb.cern.ch
// 
// Sergey Belov <Sergey.Belov@cern.ch>, 2007
//

#include <sstream>
#include <string>

extern "C" {
    #include "lhaef.h"
}

#include "hepml.hpp"
                 

extern "C" void * init_lhaef_document (void)
{
    return new mcdb::Article();
}


namespace hepml {

    extern std::string articleToXml(mcdb::Article&, int indentLevel=0, bool addXmlHeader=false);

} // namespase hepml


extern "C" int form_hepml_document(void * doc, char ** hepml_doc, int indentLevel, int addXmlHeader)
{
    if ( ! doc ) { return WRONG_INPUT_DATA; }
        
    mcdb::Article * a;
    try {
        a = reinterpret_cast<mcdb::Article*>(doc);
    } catch(...) {
        return WRONG_DOCUMENT;
    }

    std::string xml;
    
    xml = hepml::articleToXml( *a, indentLevel, (addXmlHeader !=0) );
    std::size_t length = strlen( xml.c_str() ) + 1;
    
    *hepml_doc = new char[length];
    strcpy( *hepml_doc, xml.c_str() );
    
    return length;
}


extern "C" void release_hepml_document(char * doc)
{
    delete doc;
}

extern "C" void release_lhaef_document(void * doc)
{
    mcdb::Article * a = (mcdb::Article*)doc;
    delete a;
}



/*
* Auxiliary functions.
* Are accessible only in this module
*/
namespace {
    int checkInputData(void * doc, void * data)
    {
        if ( ! ( doc && data ) ) { return WRONG_INPUT_DATA; }
        
        try {
            mcdb::Article * a = reinterpret_cast<mcdb::Article*>(doc);
        } catch(...) {
            return WRONG_DOCUMENT;
        }
        
        return 1;
    }

    /**
    * Converts lhaef_math_notation to mcdb::MathNotation.
    * If input pointer iz zero, returns empty mcdb::MathNotation
    */
    mcdb::MathNotation toMathNotation(lhaef_math_notation * n)
    {
        mcdb::MathNotation notation;
        
        if ( !n ) {
            return notation;
        }
        
        notation.plain  = n->plain;
        notation.html   = n->html;
        notation.latex  = n->latex;
        notation.mathml = n->mathml;
        
        return notation;
    }
    
    
    mcdb::CrossSection toCrossSection( double cs,
                                       double csErrorPlus,
                                       double csErrorMinus,
                                       enum cross_section_type csUnit)
    {
        mcdb::CrossSection csec;
        csec.value      = cs;
        csec.errorPlus  = csErrorPlus;
        csec.errorMinus = csErrorMinus;
        csec.unit       = (mcdb::CrossSection::Unit) csUnit;
        
        return csec;
    }
    
} // anonymous namespace




/** Converts value of any type to string
 * @param t value of template type T
 * @return std::string contains transformed value
 */
template <class T>
        inline std::string toString(const T& t)
{
    std::stringstream ss;
    ss << t;
    return ss.str();
}


/*
* Returns 1 on success, 0 if input data is completely wrong
* and -1 if input data are not complete
*/
extern "C" int set_general_description(void * doc, lhaef_general_description * sample)
{
    int check = checkInputData( doc, sample );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;
    a->title()    = sample->title;
    a->abstract() = sample->abstract;
    a->comments() = sample->comments;
    
    lhapdf_experiment * exp = sample->experiment;
    
    if ( ! exp ) { return INCOMPLETE_INPUT_DATA; }
    mcdb::ExperimentGroup e ( exp->experiment, exp->group, exp->description );
    e.responsiblePerson = exp->responsiblePerson;
    
    a->experimentGroup() = e;
    
    return 1;
}



extern "C" int add_file(void * doc, lhaef_file * file)
{
    int check = checkInputData( doc, file );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;
    
    mcdb::CrossSection   csec;
    mcdb::File::Checksum csum;
    
    csec = toCrossSection( file->cross_section, file->cross_section_upper_error,
                           file->cross_section_lower_error, file->cross_section_unit);
    
    csum.type  = (mcdb::File::Checksum::Type)file->checksum_type;
    csum.value = file->checksum;
    
    mcdb::File f;
        
    f.paths().push_back( file->location_path );
    f.type()         = (mcdb::File::Type)file->type;
    f.eventsNumber() = file->n_events;
    f.size()         = file->size;
    f.checksum()     = csum;
    f.crossSection() = csec;
    
    a->files().push_back( f );
    
    
    return a->files().size();
}


extern "C" int set_generator(void * doc, lhaef_generator * gen)
{
    int check = checkInputData( doc, gen );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;

    mcdb::Generator &g = a->generator();
    g.name()        = gen->name;
    g.version()     = gen->version;
    g.homepage()    = gen->homepage;
    g.description() = gen->description;
    
    
    return 1;
}


extern "C" int set_model(void * doc, lhaef_model * model)
{
    int check = checkInputData( doc, model );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;
    
    mcdb::Model &m  = a->model();
    m.name()        = model->name;
    m.description() = model->description;
    
    return 1;
}


extern "C" int add_model_parameter(void * doc, lhaef_parameter * par)
{
    int check = checkInputData( doc, par );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;

    mcdb::Model::Parameter p ( par->name, par->value );
    p.description()  = par->description;
    p.mathNotation() = toMathNotation( par->notation );
    
    a->model().parameters().push_back( p );
    
    return a->model().parameters().size();
}


extern "C" int add_cut(void * doc, lhaef_cut * cut, int cutsetId)
{
    int check = checkInputData( doc, cut );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;
    
    mcdb::Cut c;
    c.object()         = cut->name;
    c.objectNotation() = toMathNotation( cut->object_notation );
    c.minValue()       = cut->min_value;
    c.maxValue()       = cut->max_value;
    c.logic()          = (mcdb::Cut::Logic) cut->cut_logic;
    c.cutsetId()       = cutsetId;
    
    a->cuts().push_back( c );
    
    return a->cuts().size();
}


extern "C" int add_author(void * doc, lhaef_author * author)
{
    int check = checkInputData( doc, author );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;
    
    mcdb::Author auth;
    auth.firstName()  = author->firstName;
    auth.lastName()   = author->lastName;
    auth.email()      = author->email;
    auth.experiment() = author->experiment;
    auth.experimentGroup() = author->group;
    auth.organization()    = author->organization;

    a->authors().push_back( auth );
    
    return a->authors().size();
}


extern "C" int set_process(void * doc, lhaef_process * proc)
{
    int check = checkInputData( doc, proc );
    if ( check <= 0  ) { return check; }
    
    mcdb::Article * a = (mcdb::Article*)doc;
    
    mcdb::Process &p = a->process();
    
    p.finalState() = proc->final_state;
    p.finalStateNotation() = toMathNotation( proc->final_state_notation );
    
    p.crossSection() = toCrossSection ( proc->cross_section,
                                      proc->cross_section_upper_error,
                                      proc->cross_section_lower_error,
                                      proc->cross_section_unit );
    return 1;
}

namespace {
    int getBeamReference(void * doc, int i, mcdb::Beam * &beam ) 
    {
        if ( i<0 || i>1 ) {
            return WRONG_INPUT_DATA;
        }
    
        mcdb::Article * a = (mcdb::Article*)doc;
    
        if ( i == 0 ) {
            beam = &a->process().beam1();
        } else {
            beam = &a->process().beam2();
        }
        
        return 1;
    }
}

extern "C" int set_process_beam(void * doc, int i, lhaef_beam * beam)
{
    int check = checkInputData( doc, beam );
    if ( check <= 0  ) { return check; }
    
    mcdb::Beam * b;
    int beamGettingResult = getBeamReference(doc, i, b);
    if ( beamGettingResult <= 0  ) { return beamGettingResult; }
    
    b->particle.name   = beam->particle_name;
    b->particle.kfCode = beam->particle_pdgcode;
    b->energy.value = beam->energy;
    b->energy.unit  = (mcdb::Beam::Energy::Unit) beam->energy_unit;
    
    return 1;
}



extern "C" int set_process_beam_pdf(void * doc, int i, lhaef_pdf * pdf)
{
    int check = checkInputData( doc, pdf );
    if ( check <= 0  ) { return check; }
    
    mcdb::Beam * b;
    int beamGettingResult = getBeamReference(doc, i, b);
    if ( beamGettingResult <= 0  ) { return beamGettingResult; }
    
    mcdb::Pdf &p = b->pdf;
    
    p.name         = pdf->name;
    p.version      = pdf->version;
    p.pdfLibSet    = pdf->pdflib_set;
    p.pdfLibGroup  = pdf->pdflib_group;
    p.lhaPdfSet    = pdf->lhapdf_set;
    p.lhaPdfMember = pdf->lhapdf_member;
    p.lhaPdfFile   = pdf->lhapdf_filename;
    
    return 1;
}

extern "C" int set_process_beam_pdf_alphas(void * doc, int i, lhaef_alphas * alphas)
{
    int check = checkInputData( doc, alphas );
    if ( check <= 0  ) { return check; }
    
    mcdb::Beam * b;
    int beamGettingResult = getBeamReference(doc, i, b);
    if ( beamGettingResult <= 0  ) { return beamGettingResult; }
    
    mcdb::QcdCoupling &c = b->qcdCoupling;
    
    c.lambda      = alphas->lambda;
    c.nFlavours   = alphas->nflavours;
    c.nLoopsAlpha = alphas->nloops;
    
    return 1;
}

extern "C" int set_process_alphas(void * doc, lhaef_alphas * alphas)
{
    int check = checkInputData( doc, alphas );
    if ( check <= 0  ) { return check; }

    mcdb::Article * a = (mcdb::Article*)doc;
    mcdb::QcdCoupling &c = a->process().qcdCoupling();
    
    c.lambda      = alphas->lambda;
    c.nFlavours   = alphas->nflavours;
    c.nLoopsAlpha = alphas->nloops;
    
    return 1;

}

extern "C" int add_subprocess(void * doc, lhaef_subprocess * subproc, int cutsetId)
{
    int check = checkInputData( doc, subproc );
    if ( check <= 0  ) { return check; }
    mcdb::Article * a = (mcdb::Article*)doc;
    
    mcdb::Subprocess s;
    s.notation() = subproc->name;
    s.crossSection() = toCrossSection ( subproc->cross_section,
                                        subproc->cross_section_upper_error,
                                        subproc->cross_section_lower_error,
                                        subproc->cross_section_unit );
    s.factorisationScale()   = toMathNotation( subproc->factorisationScale );
    s.renormalisationScale() = toMathNotation( subproc->renormalisationScale );
    s.cutsetId() = cutsetId;
    
    a->process().subprocesses().push_back( s );
    
    return a->process().subprocesses().size();
}



